@@ -242,7 +242,7 @@ _timeline_track_added_cb (GESTimeline * timeline, GESTrack * track,
     GESPipeline * pipeline)
 {
   track_disable_last_gap (track,
-      ! !(pipeline->priv->mode & (GES_PIPELINE_MODE_RENDER |
+      !!(pipeline->priv->mode & (GES_PIPELINE_MODE_RENDER |
               GES_PIPELINE_MODE_SMART_RENDER)));
   _link_track (pipeline, track);
 }
@@ -254,6 +254,17 @@ _timeline_track_removed_cb (GESTimeline * timeline, GESTrack * track,
   _unlink_track (pipeline, track);
 }
 
+
+static void
+ges_pipeline_constructed (GObject * object)
+{
+  GESPipeline *self = GES_PIPELINE (object);
+
+  ges_pipeline_set_mode (self, DEFAULT_TIMELINE_MODE);
+
+  ((GObjectClass *) ges_pipeline_parent_class)->constructed (object);
+}
+
 static void
 ges_pipeline_dispose (GObject * object)
 {
@@ -301,6 +312,8 @@ ges_pipeline_class_init (GESPipelineClass * klass)
   GST_DEBUG_CATEGORY_INIT (ges_pipeline_debug, "gespipeline",
       GST_DEBUG_FG_YELLOW, "ges pipeline");
 
+
+  object_class->constructed = ges_pipeline_constructed;
   object_class->dispose = ges_pipeline_dispose;
   object_class->get_property = ges_pipeline_get_property;
   object_class->set_property = ges_pipeline_set_property;
@@ -397,7 +410,8 @@ ges_pipeline_init (GESPipeline * self)
   self->priv->playsink =
       gst_element_factory_make ("playsink", "internal-sinks");
   self->priv->encodebin =
-      gst_element_factory_make ("encodebin", "internal-encodebin");
+      gst_element_factory_make ("encodebin2", "internal-encodebin");
+
   g_object_set (self->priv->encodebin, "avoid-reencoding", TRUE, NULL);
 
   if (G_UNLIKELY (self->priv->playsink == NULL))
@@ -405,8 +419,6 @@ ges_pipeline_init (GESPipeline * self)
   if (G_UNLIKELY (self->priv->encodebin == NULL))
     goto no_encodebin;
 
-  ges_pipeline_set_mode (self, DEFAULT_TIMELINE_MODE);
-
   return;
 
 no_playsink:
@@ -505,7 +517,8 @@ _link_tracks (GESPipeline * pipeline)
 {
   GList *tmp;
 
-  GST_DEBUG_OBJECT (pipeline, "Linking tracks");
+  GST_DEBUG_OBJECT (pipeline, "Linking %d tracks",
+      g_list_length (pipeline->priv->timeline->tracks));
 
   if (!pipeline->priv->timeline) {
     GST_INFO_OBJECT (pipeline, "Not timeline set yet, doing nothing");
@@ -616,8 +629,8 @@ ges_pipeline_change_state (GstElement * element, GstStateChange transition)
 
       for (tmp = self->priv->not_rendered_tracks; tmp; tmp = tmp->next)
         gst_element_set_locked_state (tmp->data, FALSE);
-    }
       break;
+    }
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
     {
       GstElement *queue = gst_bin_get_by_name (GST_BIN (self->priv->playsink),
@@ -1208,7 +1221,7 @@ ges_pipeline_set_mode (GESPipeline * pipeline, GESPipelineFlags mode)
 
   if (pipeline->priv->timeline) {
     gboolean disabled =
-        ! !(mode & (GES_PIPELINE_MODE_RENDER | GES_PIPELINE_MODE_SMART_RENDER));
+        !!(mode & (GES_PIPELINE_MODE_RENDER | GES_PIPELINE_MODE_SMART_RENDER));
 
     for (tmp = pipeline->priv->timeline->tracks; tmp; tmp = tmp->next)
       track_disable_last_gap (GES_TRACK (tmp->data), disabled);
@@ -1256,19 +1269,33 @@ ges_pipeline_set_mode (GESPipeline * pipeline, GESPipelineFlags mode)
       GST_ERROR_OBJECT (pipeline, "Output URI not set !");
       return FALSE;
     }
+
     if (!gst_bin_add (GST_BIN_CAST (pipeline), pipeline->priv->encodebin)) {
       GST_ERROR_OBJECT (pipeline, "Couldn't add encodebin");
       return FALSE;
     }
-    if (!gst_bin_add (GST_BIN_CAST (pipeline), pipeline->priv->urisink)) {
-      GST_ERROR_OBJECT (pipeline, "Couldn't add URI sink");
-      return FALSE;
+
+    /* We know that encodebin2 will add its src pads as soon as the profile is
+     * set, so link the sink to it if it has a pad */
+    if (pipeline->priv->encodebin->numsrcpads > 0) {
+      if (pipeline->priv->encodebin->numsrcpads > 1) {
+        GST_WARNING_OBJECT (pipeline,
+            "Encodebin has more than one srcpad, this is not supported yet");
+      }
+
+      if (!gst_bin_add (GST_BIN_CAST (pipeline), pipeline->priv->urisink)) {
+        GST_ERROR_OBJECT (pipeline, "Couldn't add URI sink");
+        return FALSE;
+      }
+
+      gst_element_link_pads_full (pipeline->priv->encodebin, "src_0",
+          pipeline->priv->urisink, "sink", GST_PAD_LINK_CHECK_NOTHING);
+    } else {
+      GST_INFO_OBJECT (pipeline,
+          "Using an muxing sink, not adding any sink element");
     }
     g_object_set (pipeline->priv->encodebin, "avoid-reencoding",
         !(!(mode & GES_PIPELINE_MODE_SMART_RENDER)), NULL);
-
-    gst_element_link_pads_full (pipeline->priv->encodebin, "src",
-        pipeline->priv->urisink, "sink", GST_PAD_LINK_CHECK_NOTHING);
   }
 
   if (pipeline->priv->timeline) {
