@@ -26,6 +26,7 @@
 #include <gst/video/video.h>
 
 #include "gstframepositioner.h"
+#include "ges-frame-composition-meta.h"
 #include "ges-internal.h"
 
 GST_DEBUG_CATEGORY_STATIC (_framepositioner);
@@ -43,21 +44,24 @@ static void gst_frame_positioner_get_property (GObject * object,
 static GstFlowReturn gst_frame_positioner_transform_ip (GstBaseTransform *
     trans, GstBuffer * buf);
 
-static gboolean
-gst_frame_positioner_meta_init (GstMeta * meta, gpointer params,
-    GstBuffer * buffer);
-static gboolean gst_frame_positioner_meta_transform (GstBuffer * dest,
-    GstMeta * meta, GstBuffer * buffer, GQuark type, gpointer data);
-
 enum
 {
   PROP_0,
   PROP_ALPHA,
+
   PROP_POSX,
+  PROP_FPOSX,
+
   PROP_POSY,
-  PROP_ZORDER,
+  PROP_FPOSY,
+
   PROP_WIDTH,
+  PROP_FWIDTH,
+
   PROP_HEIGHT,
+  PROP_FHEIGHT,
+
+  PROP_ZORDER,
   PROP_OPERATOR,
   PROP_LAST,
 };
@@ -118,6 +122,24 @@ gst_compositor_operator_get_type_and_default_value (int *default_operator_value)
   return operator_gtype;
 }
 
+static gboolean
+scales_downstream (GstFramePositioner * self)
+{
+  if (self->scale_in_compositor)
+    return TRUE;
+
+  if (!self->track_source)
+    return self->scale_in_compositor;
+
+  GESTimelineElement *parent = GES_TIMELINE_ELEMENT_PARENT (self->track_source);
+
+  if (!parent || !GES_IS_CLIP (parent)) {
+    return self->scale_in_compositor;
+  }
+
+  return ges_clip_has_scale_effect (GES_CLIP (parent));
+}
+
 static void
 _weak_notify_cb (GstFramePositioner * pos, GObject * old)
 {
@@ -130,9 +152,16 @@ is_user_positionned (GstFramePositioner * self)
   gint i;
   GParamSpec *positioning_props[] = {
     properties[PROP_WIDTH],
+    properties[PROP_FWIDTH],
+
     properties[PROP_HEIGHT],
+    properties[PROP_FHEIGHT],
+
     properties[PROP_POSX],
+    properties[PROP_FPOSX],
+
     properties[PROP_POSY],
+    properties[PROP_FPOSY],
   };
 
   if (self->user_positioned)
@@ -206,13 +235,22 @@ reposition_properties (GstFramePositioner * pos, gint old_track_width,
     gint old_track_height)
 {
   gint i;
+  /* *INDENT-OFF* */
   RepositionPropertyData props_data[] = {
+    {&pos->width, old_track_width, pos->track_width, properties[PROP_FWIDTH]},
     {&pos->width, old_track_width, pos->track_width, properties[PROP_WIDTH]},
-    {&pos->height, old_track_height, pos->track_height,
-        properties[PROP_HEIGHT]},
+
+    {&pos->height, old_track_height, pos->track_height, properties[PROP_FHEIGHT]},
+    {&pos->height, old_track_height, pos->track_height, properties[PROP_HEIGHT]},
+
+    {&pos->posx, old_track_width, pos->track_width, properties[PROP_FPOSX]},
     {&pos->posx, old_track_width, pos->track_width, properties[PROP_POSX]},
+
+    {&pos->posy, old_track_height, pos->track_height, properties[PROP_FPOSY]},
     {&pos->posy, old_track_height, pos->track_height, properties[PROP_POSY]},
   };
+  /* *INDENT-ON* */
+
 
   for (i = 0; i < G_N_ELEMENTS (props_data); i++) {
     GList *values, *tmp;
@@ -223,8 +261,10 @@ reposition_properties (GstFramePositioner * pos, gint old_track_width,
     GstControlBinding *binding =
         gst_object_get_control_binding (GST_OBJECT (pos), d.pspec->name);
 
-    *(d.value) =
-        *(d.value) * (gdouble) d.track_value / (gdouble) d.old_track_value;
+    if (G_PARAM_SPEC_VALUE_TYPE (d.pspec) == G_TYPE_FLOAT) {
+      *(d.value) =
+          *(d.value) * (gdouble) d.track_value / (gdouble) d.old_track_value;
+    }
 
     if (!binding)
       continue;
@@ -279,7 +319,7 @@ gst_frame_positioner_update_properties (GstFramePositioner * pos,
   caps = gst_caps_from_string ("video/x-raw(ANY)");
 
   if (pos->track_width && pos->track_height &&
-      (!track_mixing || !pos->scale_in_compositor)) {
+      (!track_mixing || !scales_downstream (pos))) {
     gst_caps_set_simple (caps, "width", G_TYPE_INT,
         pos->track_width, "height", G_TYPE_INT, pos->track_height, NULL);
   }
@@ -326,6 +366,13 @@ gst_frame_positioner_update_properties (GstFramePositioner * pos,
   reposition_properties (pos, old_track_width, old_track_height);
 
 done:
+  if (scales_downstream (pos) && pos->natural_width && pos->natural_height) {
+    GST_DEBUG_OBJECT (pos,
+        "Forcing natural width in source make downstream scaling work");
+    gst_caps_set_simple (caps, "width", G_TYPE_INT, pos->natural_width,
+        "height", G_TYPE_INT, pos->natural_height, NULL);
+  }
+
   GST_DEBUG_OBJECT (pos, "setting caps %" GST_PTR_FORMAT, caps);
 
   g_object_set (pos->capsfilter, "caps", caps, NULL);
@@ -474,7 +521,7 @@ gst_frame_positioner_class_init (GstFramePositionerClass * klass)
   GstBaseTransformClass *base_transform_class =
       GST_BASE_TRANSFORM_CLASS (klass);
 
-  GST_DEBUG_CATEGORY_INIT (_framepositioner, "framepositioner",
+  GST_DEBUG_CATEGORY_INIT (_framepositioner, "gesframepositioner",
       GST_DEBUG_FG_YELLOW, "ges frame positioner");
 
   gst_element_class_add_static_pad_template (GST_ELEMENT_CLASS (klass),
@@ -503,9 +550,19 @@ gst_frame_positioner_class_init (GstFramePositionerClass * klass)
    * The desired x position for the stream.
    */
   properties[PROP_POSX] =
-      g_param_spec_int ("posx", "posx", "x position of the stream", MIN_PIXELS,
-      MAX_PIXELS, 0, G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE);
+      g_param_spec_int ("posx", "posx", "x position of the stream",
+      MIN_PIXELS, MAX_PIXELS, 0,
+      G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_LAX_VALIDATION);
 
+  /**
+   * gstframepositioner:fposx:
+   *
+   * The desired x position for the stream.
+   */
+  properties[PROP_FPOSX] =
+      g_param_spec_float ("fposx", "fposx", "x position of the stream in float",
+      MIN_PIXELS, MAX_PIXELS, 0,
+      G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_LAX_VALIDATION);
 
   /**
    * gstframepositioner:posy:
@@ -513,8 +570,20 @@ gst_frame_positioner_class_init (GstFramePositionerClass * klass)
    * The desired y position for the stream.
    */
   properties[PROP_POSY] =
-      g_param_spec_int ("posy", "posy", "y position of the stream", MIN_PIXELS,
-      MAX_PIXELS, 0, G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE);
+      g_param_spec_int ("posy", "posy", "y position of the stream",
+      MIN_PIXELS, MAX_PIXELS, 0,
+      G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_LAX_VALIDATION);
+
+
+  /**
+   * gstframepositioner:fposy:
+   *
+   * The desired y position for the stream.
+   */
+  properties[PROP_FPOSY] =
+      g_param_spec_float ("fposy", "fposy", "y position of the stream in float",
+      MIN_PIXELS, MAX_PIXELS, 0,
+      G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_LAX_VALIDATION);
 
   /**
    * gstframepositioner:zorder:
@@ -532,8 +601,20 @@ gst_frame_positioner_class_init (GstFramePositionerClass * klass)
    * Set to 0 if size is not mandatory, will be set to width of the current track.
    */
   properties[PROP_WIDTH] =
-      g_param_spec_int ("width", "width", "width of the source", 0, MAX_PIXELS,
-      0, G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE);
+      g_param_spec_int ("width", "width", "width of the source", 0,
+      MAX_PIXELS, 0,
+      G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_LAX_VALIDATION);
+
+  /**
+   * gesframepositioner:fwidth:
+   *
+   * The desired width for that source.
+   * Set to 0 if size is not mandatory, will be set to width of the current track.
+   */
+  properties[PROP_FWIDTH] =
+      g_param_spec_float ("fwidth", "fwidth", "width of the source in float", 0,
+      MAX_PIXELS, 0,
+      G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_LAX_VALIDATION);
 
   /**
    * gesframepositioner:height:
@@ -543,7 +624,19 @@ gst_frame_positioner_class_init (GstFramePositionerClass * klass)
    */
   properties[PROP_HEIGHT] =
       g_param_spec_int ("height", "height", "height of the source", 0,
-      MAX_PIXELS, 0, G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE);
+      MAX_PIXELS, 0,
+      G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_LAX_VALIDATION);
+
+  /**
+   * gesframepositioner:fheight:
+   *
+   * The desired height for that source.
+   * Set to 0 if size is not mandatory, will be set to height of the current track.
+   */
+  properties[PROP_FHEIGHT] =
+      g_param_spec_float ("fheight", "fheight", "height of the source in float",
+      0, MAX_PIXELS, 0,
+      G_PARAM_READWRITE | GST_PARAM_CONTROLLABLE | G_PARAM_LAX_VALIDATION);
 
   /**
    * gesframepositioner:operator:
@@ -615,10 +708,18 @@ gst_frame_positioner_set_property (GObject * object, guint property_id,
       framepositioner->posx = g_value_get_int (value);
       framepositioner->user_positioned = TRUE;
       break;
+    case PROP_FPOSX:
+      framepositioner->posx = g_value_get_float (value);
+      framepositioner->user_positioned = TRUE;
+      break;
     case PROP_POSY:
       framepositioner->posy = g_value_get_int (value);
       framepositioner->user_positioned = TRUE;
       break;
+    case PROP_FPOSY:
+      framepositioner->posy = g_value_get_float (value);
+      framepositioner->user_positioned = TRUE;
+      break;
     case PROP_ZORDER:
       framepositioner->zorder = g_value_get_uint (value);
       break;
@@ -628,12 +729,24 @@ gst_frame_positioner_set_property (GObject * object, guint property_id,
       gst_frame_positioner_update_properties (framepositioner, track_mixing,
           0, 0);
       break;
+    case PROP_FWIDTH:
+      framepositioner->user_positioned = TRUE;
+      framepositioner->width = g_value_get_float (value);
+      gst_frame_positioner_update_properties (framepositioner, track_mixing,
+          0, 0);
+      break;
     case PROP_HEIGHT:
       framepositioner->user_positioned = TRUE;
       framepositioner->height = g_value_get_int (value);
       gst_frame_positioner_update_properties (framepositioner, track_mixing,
           0, 0);
       break;
+    case PROP_FHEIGHT:
+      framepositioner->user_positioned = TRUE;
+      framepositioner->height = g_value_get_float (value);
+      gst_frame_positioner_update_properties (framepositioner, track_mixing,
+          0, 0);
+      break;
     case PROP_OPERATOR:
       framepositioner->operator = g_value_get_enum (value);
       gst_frame_positioner_update_properties (framepositioner, track_mixing,
@@ -651,7 +764,7 @@ gst_frame_positioner_get_property (GObject * object, guint property_id,
     GValue * value, GParamSpec * pspec)
 {
   GstFramePositioner *pos = GST_FRAME_POSITIONNER (object);
-  gint real_width, real_height;
+  gdouble real_width, real_height;
 
   switch (property_id) {
     case PROP_ALPHA:
@@ -660,9 +773,15 @@ gst_frame_positioner_get_property (GObject * object, guint property_id,
     case PROP_POSX:
       g_value_set_int (value, round (pos->posx));
       break;
+    case PROP_FPOSX:
+      g_value_set_float (value, pos->posx);
+      break;
     case PROP_POSY:
       g_value_set_int (value, round (pos->posy));
       break;
+    case PROP_FPOSY:
+      g_value_set_float (value, pos->posy);
+      break;
     case PROP_ZORDER:
       g_value_set_uint (value, pos->zorder);
       break;
@@ -670,18 +789,32 @@ gst_frame_positioner_get_property (GObject * object, guint property_id,
       if (pos->scale_in_compositor) {
         g_value_set_int (value, round (pos->width));
       } else {
-        real_width =
-            pos->width > 0 ? round (pos->width) : round (pos->track_width);
-        g_value_set_int (value, real_width);
+        real_width = pos->width > 0 ? pos->width : pos->track_width;
+        g_value_set_int (value, round (real_width));
+      }
+      break;
+    case PROP_FWIDTH:
+      if (pos->scale_in_compositor) {
+        g_value_set_float (value, pos->width);
+      } else {
+        real_width = pos->width > 0 ? pos->width : pos->track_width;
+        g_value_set_float (value, real_width);
       }
       break;
     case PROP_HEIGHT:
       if (pos->scale_in_compositor) {
         g_value_set_int (value, round (pos->height));
       } else {
-        real_height =
-            pos->height > 0 ? round (pos->height) : round (pos->track_height);
-        g_value_set_int (value, real_height);
+        real_height = pos->height > 0 ? pos->height : pos->track_height;
+        g_value_set_int (value, round (real_height));
+      }
+      break;
+    case PROP_FHEIGHT:
+      if (pos->scale_in_compositor) {
+        g_value_set_float (value, pos->height);
+      } else {
+        real_height = pos->height > 0 ? pos->height : pos->track_height;
+        g_value_set_float (value, real_height);
       }
       break;
     case PROP_OPERATOR:
@@ -693,84 +826,10 @@ gst_frame_positioner_get_property (GObject * object, guint property_id,
   }
 }
 
-GType
-gst_frame_positioner_meta_api_get_type (void)
-{
-  static GType type;
-  static const gchar *tags[] = { "video", NULL };
-
-  if (g_once_init_enter (&type)) {
-    GType _type = gst_meta_api_type_register ("GstFramePositionerApi", tags);
-    g_once_init_leave (&type, _type);
-  }
-  return type;
-}
-
-static const GstMetaInfo *
-gst_frame_positioner_get_info (void)
-{
-  static const GstMetaInfo *meta_info = NULL;
-
-  if (g_once_init_enter ((GstMetaInfo **) & meta_info)) {
-    const GstMetaInfo *meta =
-        gst_meta_register (gst_frame_positioner_meta_api_get_type (),
-        "GstFramePositionerMeta",
-        sizeof (GstFramePositionerMeta), gst_frame_positioner_meta_init,
-        NULL,
-        gst_frame_positioner_meta_transform);
-    g_once_init_leave ((GstMetaInfo **) & meta_info, (GstMetaInfo *) meta);
-  }
-  return meta_info;
-}
-
-static gboolean
-gst_frame_positioner_meta_init (GstMeta * meta, gpointer params,
-    GstBuffer * buffer)
-{
-  int default_operator_value = 0;
-  GstFramePositionerMeta *smeta;
-
-  smeta = (GstFramePositionerMeta *) meta;
-
-  gst_compositor_operator_get_type_and_default_value (&default_operator_value);
-
-  smeta->alpha = 0.0;
-  smeta->posx = smeta->posy = smeta->height = smeta->width = 0;
-  smeta->zorder = 0;
-  smeta->operator = default_operator_value;
-
-  return TRUE;
-}
-
-static gboolean
-gst_frame_positioner_meta_transform (GstBuffer * dest, GstMeta * meta,
-    GstBuffer * buffer, GQuark type, gpointer data)
-{
-  GstFramePositionerMeta *dmeta, *smeta;
-
-  smeta = (GstFramePositionerMeta *) meta;
-
-  if (GST_META_TRANSFORM_IS_COPY (type)) {
-    /* only copy if the complete data is copied as well */
-    dmeta =
-        (GstFramePositionerMeta *) gst_buffer_add_meta (dest,
-        gst_frame_positioner_get_info (), NULL);
-    dmeta->alpha = smeta->alpha;
-    dmeta->posx = smeta->posx;
-    dmeta->posy = smeta->posy;
-    dmeta->width = smeta->width;
-    dmeta->height = smeta->height;
-    dmeta->zorder = smeta->zorder;
-    dmeta->operator = smeta->operator;
-  }
-
-  return TRUE;
-}
-
 static GstFlowReturn
 gst_frame_positioner_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
 {
-  GstFramePositionerMeta *meta;
+  GESFrameCompositionMeta *meta;
   GstFramePositioner *framepositioner = GST_FRAME_POSITIONNER (trans);
   GstClockTime timestamp = GST_BUFFER_PTS (buf);
 
@@ -778,19 +837,58 @@ gst_frame_positioner_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
     gst_object_sync_values (GST_OBJECT (trans), timestamp);
   }
 
-  meta =
-      (GstFramePositionerMeta *) gst_buffer_add_meta (buf,
-      gst_frame_positioner_get_info (), NULL);
+  meta = ges_buffer_add_frame_composition_meta (buf);
 
   GST_OBJECT_LOCK (framepositioner);
   meta->alpha = framepositioner->alpha;
-  meta->posx = round (framepositioner->posx);
-  meta->posy = round (framepositioner->posy);
-  meta->width = round (framepositioner->width);
-  meta->height = round (framepositioner->height);
+  meta->posx = framepositioner->posx;
+  meta->posy = framepositioner->posy;
+  meta->width = framepositioner->width;
+  meta->height = framepositioner->height;
   meta->zorder = framepositioner->zorder;
   meta->operator = framepositioner->operator;
   GST_OBJECT_UNLOCK (framepositioner);
 
   return GST_FLOW_OK;
 }
+
+gboolean
+gst_frame_positioner_check_can_add_binding (GstFramePositioner * self,
+    const gchar * property_name)
+{
+  gint i = 0;
+  const gchar *checked_prop = NULL;
+  const gchar *props[][2] = {
+    {"posx", "fposx"},
+    {"posy", "fposy"},
+    {"width", "fwidth"},
+    {"height", "fheight"},
+  };
+
+
+  for (i = 0; i < G_N_ELEMENTS (props); i++) {
+    if (!g_strcmp0 (property_name, props[i][0])) {
+      checked_prop = props[i][1];
+      break;
+    } else if (!g_strcmp0 (property_name, props[i][1])) {
+      checked_prop = props[i][0];
+      break;
+    }
+  }
+
+  if (!checked_prop)
+    return TRUE;
+
+  GstControlBinding *b =
+      gst_object_get_control_binding (GST_OBJECT (self), checked_prop);
+  if (b) {
+    gst_object_unref (b);
+    GST_WARNING_OBJECT (self,
+        "Can't add control binding for %s as %s already has one", property_name,
+        checked_prop);
+
+    return FALSE;
+  }
+
+  return TRUE;
+}
