@@ -589,7 +589,7 @@ ghostpad_query_function (GstPad * ghostpad, GstObject * parent,
 static void
 internal_pad_finalizing (NlePadPrivate * priv, GObject * pad G_GNUC_UNUSED)
 {
-  g_slice_free (NlePadPrivate, priv);
+  g_free (priv);
 }
 
 static inline GstPad *
@@ -630,7 +630,7 @@ control_internal_pad (GstPad * ghostpad, NleObject * object)
   if (G_UNLIKELY (!(priv = gst_pad_get_element_private (internal)))) {
     GST_DEBUG_OBJECT (internal,
         "Creating a NlePadPrivate to put in element_private");
-    priv = g_slice_new0 (NlePadPrivate);
+    priv = g_new0 (NlePadPrivate, 1);
 
     /* Remember existing pad functions */
     priv->eventfunc = GST_PAD_EVENTFUNC (internal);
@@ -729,7 +729,7 @@ nle_object_ghost_pad_no_target (NleObject * object, const gchar * name,
 
 
   /* remember the existing ghostpad event/query/link/unlink functions */
-  priv = g_slice_new0 (NlePadPrivate);
+  priv = g_new0 (NlePadPrivate, 1);
   priv->dir = dir;
   priv->object = object;
 
@@ -763,7 +763,7 @@ nle_object_remove_ghost_pad (NleObject * object, GstPad * ghost)
   gst_ghost_pad_set_target (GST_GHOST_PAD (ghost), NULL);
   gst_element_remove_pad (GST_ELEMENT (object), ghost);
   if (priv)
-    g_slice_free (NlePadPrivate, priv);
+    g_free (priv);
 }
 
 gboolean
